@@ -779,9 +779,13 @@ attribute_is_sensitive (const CK_ATTRIBUTE *attr,
 	X (CKA_IBM_PROTKEY_EXTRACTABLE)
 	X (CKA_IBM_PROTKEY_NEVER_EXTRACTABLE)
 	X (CKA_IBM_OPAQUE_PKEY)
+	X (CKA_IBM_DILITHIUM_MODE)
 	X (CKA_IBM_DILITHIUM_KEYFORM)
 	X (CKA_IBM_DILITHIUM_RHO)
 	X (CKA_IBM_DILITHIUM_T1)
+	X (CKA_IBM_KYBER_MODE)
+	X (CKA_IBM_KYBER_PK)
+	X (CKA_IBM_KYBER_KEYFORM)
 	case CKA_VALUE:
 		return (klass != CKO_CERTIFICATE &&
 			klass != CKO_X_CERTIFICATE_EXTENSION);
