@@ -59,6 +59,7 @@ const p11_constant p11_constant_types[] = {
 	CT (CKA_TOKEN, "token")
 	CT (CKA_PRIVATE, "private")
 	CT (CKA_LABEL, "label")
+	CT (CKA_UNIQUE_ID, "unique-id")
 	CT (CKA_APPLICATION, "application")
 	CT (CKA_VALUE, "value")
 	CT (CKA_OBJECT_ID, "object-id")
@@ -74,6 +75,7 @@ const p11_constant p11_constant_types[] = {
 	CT (CKA_URL, "url")
 	CT (CKA_HASH_OF_SUBJECT_PUBLIC_KEY, "hash-of-subject-public-key")
 	CT (CKA_HASH_OF_ISSUER_PUBLIC_KEY, "hash-of-issuer-public-key")
+	CT (CKA_NAME_HASH_ALGORITHM, "name-hash-algorithm")
 	CT (CKA_CHECK_VALUE, "check-value")
 	CT (CKA_KEY_TYPE, "key-type")
 	CT (CKA_SUBJECT, "subject")
@@ -123,6 +125,23 @@ const p11_constant p11_constant_types[] = {
 	CT (CKA_AUTH_PIN_FLAGS, "auth-pin-flags")
 	CT (CKA_ALWAYS_AUTHENTICATE, "always-authenticate")
 	CT (CKA_WRAP_WITH_TRUSTED, "wrap-with-trusted")
+	CT (CKA_OTP_FORMAT, "otp-format")
+	CT (CKA_OTP_LENGTH, "otp-length")
+	CT (CKA_OTP_TIME_INTERVAL, "otp-time-interval")
+	CT (CKA_OTP_USER_FRIENDLY_MODE, "otp-user-friendly-mode")
+	CT (CKA_OTP_CHALLENGE_REQUIREMENT, "otp-challenge-requirement")
+	CT (CKA_OTP_TIME_REQUIREMENT, "otp-time-requirement")
+	CT (CKA_OTP_COUNTER_REQUIREMENT, "otp-counter-requirement")
+	CT (CKA_OTP_PIN_REQUIREMENT, "otp-pin-requirement")
+	CT (CKA_OTP_USER_IDENTIFIER, "otp-user-identifier")
+	CT (CKA_OTP_SERVICE_IDENTIFIER, "otp-service-identifier")
+	CT (CKA_OTP_SERVICE_LOGO, "otp-service-logo")
+	CT (CKA_OTP_SERVICE_LOGO_TYPE, "otp-service-logo-type")
+	CT (CKA_OTP_COUNTER, "otp-counter")
+	CT (CKA_OTP_TIME, "otp-time")
+	CT (CKA_GOSTR3410_PARAMS, "gostr3410-params")
+	CT (CKA_GOSTR3411_PARAMS, "gostr3411-params")
+	CT (CKA_GOST28147_PARAMS, "gost28147-params")
 	CT (CKA_HW_FEATURE_TYPE, "hw-feature-type")
 	CT (CKA_RESET_ON_INIT, "reset-on-init")
 	CT (CKA_HAS_RESET, "has-reset")
@@ -141,10 +160,38 @@ const p11_constant p11_constant_types[] = {
 	CT (CKA_DEFAULT_CMS_ATTRIBUTES, "default-cms-attributes")
 	CT (CKA_SUPPORTED_CMS_ATTRIBUTES, "supported-cms-attributes")
 	CT (CKA_PROFILE_ID, "profile-id")
+	CT (CKA_X2RATCHET_BAG, "x2ratchet-bag")
+	CT (CKA_X2RATCHET_BAGSIZE, "x2ratchet-bagsize")
+	CT (CKA_X2RATCHET_BOBS1STMSG, "x2ratchet-bobs1stmsg")
+	CT (CKA_X2RATCHET_CKR, "x2ratchet-ckr")
+	CT (CKA_X2RATCHET_CKS, "x2ratchet-cks")
+	CT (CKA_X2RATCHET_DHP, "x2ratchet-dhp")
+	CT (CKA_X2RATCHET_DHR, "x2ratchet-dhr")
+	CT (CKA_X2RATCHET_DHS, "x2ratchet-dhs")
+	CT (CKA_X2RATCHET_HKR, "x2ratchet-hkr")
+	CT (CKA_X2RATCHET_HKS, "x2ratchet-hks")
+	CT (CKA_X2RATCHET_ISALICE, "x2ratchet-isalice")
+	CT (CKA_X2RATCHET_NHKR, "x2ratchet-nhkr")
+	CT (CKA_X2RATCHET_NHKS, "x2ratchet-nhks")
+	CT (CKA_X2RATCHET_NR, "x2ratchet-nr")
+	CT (CKA_X2RATCHET_NS, "x2ratchet-ns")
+	CT (CKA_X2RATCHET_PNS, "x2ratchet-pns")
+	CT (CKA_X2RATCHET_RK, "x2ratchet-rk")
+	CT (CKA_HSS_LEVELS, "hss-levels")
+	CT (CKA_HSS_LMS_TYPE, "hss-lms-type")
+	CT (CKA_HSS_LMOTS_TYPE, "hss-lmots-type")
+	CT (CKA_HSS_LMS_TYPES, "hss-lms-types")
+	CT (CKA_HSS_LMOTS_TYPES, "hss-lmots-types")
+	CT (CKA_HSS_KEYS_REMAINING, "hss-keys-remaining")
 	CT (CKA_WRAP_TEMPLATE, "wrap-template")
 	CT (CKA_UNWRAP_TEMPLATE, "unwrap-template")
+	CT (CKA_DERIVE_TEMPLATE, "derive-template")
 	CT (CKA_ALLOWED_MECHANISMS, "allowed-mechanisms")
 	CT (CKA_IBM_OPAQUE, "ibm-opaque")
+	CT (CKA_IBM_OPAQUE_REENC, "ibm-opaque-reenc")
+	CT (CKA_IBM_OPAQUE_OLD, "ibm-opaque-old")
+	CT (CKA_IBM_KYBER_MODE, "ibm-kyber-mode")
+	CT (CKA_IBM_DILITHIUM_MODE, "ibm-dilithium-mode")
 	CT (CKA_IBM_RESTRICTABLE, "ibm-restrictable")
 	CT (CKA_IBM_NEVER_MODIFIABLE, "ibm-never-modifiable")
 	CT (CKA_IBM_RETAINKEY, "ibm-retainkey")
@@ -165,7 +212,11 @@ const p11_constant p11_constant_types[] = {
 	CT (CKA_IBM_DILITHIUM_S2, "ibm-dilithium-s2")
 	CT (CKA_IBM_DILITHIUM_T0, "ibm-dilithium-t0")
 	CT (CKA_IBM_DILITHIUM_T1, "ibm-dilithium-t1")
+	CT (CKA_IBM_KYBER_KEYFORM, "ibm-kyber-keyform")
+	CT (CKA_IBM_KYBER_PK, "ibm-kyber-pk")
+	CT (CKA_IBM_KYBER_SK, "ibm-kyber-sk")
 	CT (CKA_IBM_OPAQUE_PKEY, "ibm-opaque-pkey")
+	CT (CKA_IBM_CCA_AES_KEY_MODE, "ibm-cca-aes-key-mode")
 	CT (CKA_NSS_URL, "nss-url")
 	CT (CKA_NSS_EMAIL, "nss-email")
 	CT (CKA_NSS_SMIME_INFO, "nss-smime-constant")
@@ -273,6 +324,43 @@ const p11_constant p11_constant_keys[] = {
 	CT (CKK_AES, "aes")
 	CT (CKK_BLOWFISH, "blowfish")
 	CT (CKK_TWOFISH, "twofish")
+	CT (CKK_SECURID, "ckk-securid")
+	CT (CKK_HOTP, "ckk-hotp")
+	CT (CKK_ACTI, "ckk-acti")
+	CT (CKK_CAMELLIA, "camellia")
+	CT (CKK_ARIA, "aria")
+	CT (CKK_MD5_HMAC, "ckk-md5-hmac")
+	CT (CKK_SHA_1_HMAC, "ckk-sha-1-hmac")
+	CT (CKK_RIPEMD128_HMAC, "ckk-ripemd128-hmac")
+	CT (CKK_RIPEMD160_HMAC, "ckk-ripemd160-hmac")
+	CT (CKK_SHA256_HMAC, "ckk-sha256-hmac")
+	CT (CKK_SHA384_HMAC, "ckk-sha384-hmac")
+	CT (CKK_SHA512_HMAC, "ckk-sha512-hmac")
+	CT (CKK_SHA224_HMAC, "ckk-sha224-hmac")
+	CT (CKK_SEED, "seed")
+	CT (CKK_GOSTR3410, "ckk-gostr3410")
+	CT (CKK_GOSTR3411, "ckk-gostr3411")
+	CT (CKK_GOST28147, "ckk-gost28147")
+	CT (CKK_CHACHA20, "ckk-chacha20")
+	CT (CKK_POLY1305, "ckk-poly1305")
+	CT (CKK_AES_XTS, "ckk-aes-xts")
+	CT (CKK_SHA3_224_HMAC, "ckk-sha3-224-hmac")
+	CT (CKK_SHA3_256_HMAC, "ckk-sha3-256-hmac")
+	CT (CKK_SHA3_384_HMAC, "ckk-sha3-384-hmac")
+	CT (CKK_SHA3_512_HMAC, "ckk-sha3-512-hmac")
+	CT (CKK_BLAKE2B_160_HMAC, "ckk-blake2b-160-hmac")
+	CT (CKK_BLAKE2B_256_HMAC, "ckk-blake2b-256-hmac")
+	CT (CKK_BLAKE2B_384_HMAC, "ckk-blake2b-384-hmac")
+	CT (CKK_BLAKE2B_512_HMAC, "ckk-blake2b-512-hmac")
+	CT (CKK_SALSA20, "ckk-salsa20")
+	CT (CKK_X2RATCHET, "x2ratchet")
+	CT (CKK_EC_EDWARDS, "ec-edwards")
+	CT (CKK_EC_MONTGOMERY, "ec-montgomery")
+	CT (CKK_HKDF, "hkdf")
+	CT (CKK_SHA512_224_HMAC, "ckk-sha512-224-hmac")
+	CT (CKK_SHA512_256_HMAC, "ckk-sha512-256-hmac")
+	CT (CKK_SHA512_T_HMAC, "ckk-sha512-t-hmac")
+	CT (CKK_HSS, "ckk-hss")
 	CT (CKK_IBM_PQC_DILITHIUM, "ibm-dilithium")
 	CT (CKK_NSS_PKCS8, "nss-pkcs8")
 	{ CKA_INVALID },
@@ -324,6 +412,7 @@ const p11_constant p11_constant_returns[] = {
 	CT (CKR_ATTRIBUTE_SENSITIVE, NULL)
 	CT (CKR_ATTRIBUTE_TYPE_INVALID, NULL)
 	CT (CKR_ATTRIBUTE_VALUE_INVALID, NULL)
+	CT (CKR_ACTION_PROHIBITED, NULL)
 	CT (CKR_DATA_INVALID, NULL)
 	CT (CKR_DATA_LEN_RANGE, NULL)
 	CT (CKR_DEVICE_ERROR, NULL)
@@ -331,6 +420,7 @@ const p11_constant p11_constant_returns[] = {
 	CT (CKR_DEVICE_REMOVED, NULL)
 	CT (CKR_ENCRYPTED_DATA_INVALID, NULL)
 	CT (CKR_ENCRYPTED_DATA_LEN_RANGE, NULL)
+	CT (CKR_AEAD_DECRYPT_FAILED, NULL)
 	CT (CKR_FUNCTION_CANCELED, NULL)
 	CT (CKR_FUNCTION_NOT_PARALLEL, NULL)
 	CT (CKR_FUNCTION_NOT_SUPPORTED, NULL)
@@ -386,6 +476,7 @@ const p11_constant p11_constant_returns[] = {
 	CT (CKR_RANDOM_SEED_NOT_SUPPORTED, NULL)
 	CT (CKR_RANDOM_NO_RNG, NULL)
 	CT (CKR_DOMAIN_PARAMS_INVALID, NULL)
+	CT (CKR_CURVE_NOT_SUPPORTED, NULL)
 	CT (CKR_BUFFER_TOO_SMALL, NULL)
 	CT (CKR_SAVED_STATE_INVALID, NULL)
 	CT (CKR_INFORMATION_SENSITIVE, NULL)
@@ -394,7 +485,17 @@ const p11_constant p11_constant_returns[] = {
 	CT (CKR_CRYPTOKI_ALREADY_INITIALIZED, NULL)
 	CT (CKR_MUTEX_BAD, NULL)
 	CT (CKR_MUTEX_NOT_LOCKED, NULL)
+	CT (CKR_NEW_PIN_MODE, NULL)
+	CT (CKR_NEXT_OTP, NULL)
+	CT (CKR_EXCEEDED_MAX_ITERATIONS, NULL)
+	CT (CKR_FIPS_SELF_TEST_FAILED, NULL)
+	CT (CKR_LIBRARY_LOAD_FAILED, NULL)
+	CT (CKR_PIN_TOO_WEAK, NULL)
+	CT (CKR_PUBLIC_KEY_INVALID, NULL)
 	CT (CKR_FUNCTION_REJECTED, NULL)
+	CT (CKR_TOKEN_RESOURCE_EXCEEDED, NULL)
+	CT (CKR_OPERATION_CANCEL_FAILED, NULL)
+	CT (CKR_KEY_EXHAUSTED, NULL)
 	{ CKA_INVALID },
 };
 
@@ -417,6 +518,14 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_DSA_KEY_PAIR_GEN, "dsa-key-pair-gen")
 	CT (CKM_DSA, NULL) /* "dsa" */
 	CT (CKM_DSA_SHA1, "dsa-sha1")
+	CT (CKM_DSA_SHA224, "dsa-sha224")
+	CT (CKM_DSA_SHA256, "dsa-sha256")
+	CT (CKM_DSA_SHA384, "dsa-sha384")
+	CT (CKM_DSA_SHA512, "dsa-sha512")
+	CT (CKM_DSA_SHA3_224, "dsa-sha3-224")
+	CT (CKM_DSA_SHA3_256, "dsa-sha3-256")
+	CT (CKM_DSA_SHA3_384, "dsa-sha3-384")
+	CT (CKM_DSA_SHA3_512, "dsa-sha3-512")
 	CT (CKM_DH_PKCS_KEY_PAIR_GEN, "dh-pkcs-key-pair-gen")
 	CT (CKM_DH_PKCS_DERIVE, "dh-pkcs-derive")
 	CT (CKM_X9_42_DH_KEY_PAIR_GEN, "x9-42-dh-key-pair-gen")
@@ -429,6 +538,28 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_SHA256_RSA_PKCS_PSS, "sha256-rsa-pkcs-pss")
 	CT (CKM_SHA384_RSA_PKCS_PSS, "sha384-rsa-pkcs-pss")
 	CT (CKM_SHA512_RSA_PKCS_PSS, "sha512-rsa-pkcs-pss")
+	CT (CKM_SHA224_RSA_PKCS, "sha224-rsa-pkcs")
+	CT (CKM_SHA224_RSA_PKCS_PSS, "sha224-rsa-pkcs-pss")
+	CT (CKM_SHA512_224, "sha512-224")
+	CT (CKM_SHA512_224_HMAC, "sha512-224-hmac")
+	CT (CKM_SHA512_224_HMAC_GENERAL, "sha512-224-hmac-general")
+	CT (CKM_SHA512_224_KEY_DERIVATION, "sha512-224-key-derivation")
+	CT (CKM_SHA512_256, "sha512-256")
+	CT (CKM_SHA512_256_HMAC, "sha512-256-hmac")
+	CT (CKM_SHA512_256_HMAC_GENERAL, "sha512-256-hmac-general")
+	CT (CKM_SHA512_256_KEY_DERIVATION, "sha512-256-key-derivation")
+	CT (CKM_SHA512_T, "sha512-t")
+	CT (CKM_SHA512_T_HMAC, "sha512-t-hmac")
+	CT (CKM_SHA512_T_HMAC_GENERAL, "sha512-t-hmac-general")
+	CT (CKM_SHA512_T_KEY_DERIVATION, "sha512-t-key-derivation")
+	CT (CKM_SHA3_256_RSA_PKCS, "sha3-256-rsa-pkcs")
+	CT (CKM_SHA3_384_RSA_PKCS, "sha3-384-rsa-pkcs")
+	CT (CKM_SHA3_512_RSA_PKCS, "sha3-512-rsa-pkcs")
+	CT (CKM_SHA3_256_RSA_PKCS_PSS, "sha3-256-rsa-pkcs-pss")
+	CT (CKM_SHA3_384_RSA_PKCS_PSS, "sha3-384-rsa-pkcs-pss")
+	CT (CKM_SHA3_512_RSA_PKCS_PSS, "sha3-512-rsa-pkcs-pss")
+	CT (CKM_SHA3_224_RSA_PKCS, "sha3-224-rsa-pkcs")
+	CT (CKM_SHA3_224_RSA_PKCS_PSS, "sha3-224-rsa-pkcs-pss")
 	CT (CKM_RC2_KEY_GEN, "rc2-key-gen")
 	CT (CKM_RC2_ECB, "rc2-ecb")
 	CT (CKM_RC2_CBC, "rc2-cbc")
@@ -450,6 +581,8 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_DES3_MAC, "des3-mac")
 	CT (CKM_DES3_MAC_GENERAL, "des3-mac-general")
 	CT (CKM_DES3_CBC_PAD, "des3-cbc-pad")
+	CT (CKM_DES3_CMAC_GENERAL, "des3-cmac-general")
+	CT (CKM_DES3_CMAC, "des3-cmac")
 	CT (CKM_CDMF_KEY_GEN, "cdmf-key-gen")
 	CT (CKM_CDMF_ECB, "cdmf-ecb")
 	CT (CKM_CDMF_CBC, "cdmf-cbc")
@@ -478,12 +611,37 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_SHA256, "sha256")
 	CT (CKM_SHA256_HMAC, "sha256-hmac")
 	CT (CKM_SHA256_HMAC_GENERAL, "sha256-hmac-general")
+	CT (CKM_SHA224, "sha224")
+	CT (CKM_SHA224_HMAC, "sha224-hmac")
+	CT (CKM_SHA224_HMAC_GENERAL, "sha224-hmac-general")
 	CT (CKM_SHA384, "sha384")
 	CT (CKM_SHA384_HMAC, "sha384-hmac")
 	CT (CKM_SHA384_HMAC_GENERAL, "sha384-hmac-general")
 	CT (CKM_SHA512, "sha512")
 	CT (CKM_SHA512_HMAC, "sha512-hmac")
 	CT (CKM_SHA512_HMAC_GENERAL, "sha512-hmac-general")
+	CT (CKM_SECURID_KEY_GEN, "securid-key-gen")
+	CT (CKM_SECURID, "securid")
+	CT (CKM_HOTP_KEY_GEN, "hotp-key-gen")
+	CT (CKM_HOTP, "hotp")
+	CT (CKM_ACTI, "acti")
+	CT (CKM_ACTI_KEY_GEN, "acti-key-gen")
+	CT (CKM_SHA3_256, "sha3-256")
+	CT (CKM_SHA3_256_HMAC, "sha3-256-hmac")
+	CT (CKM_SHA3_256_HMAC_GENERAL, "sha3-256-hmac-general")
+	CT (CKM_SHA3_256_KEY_GEN, "sha3-256-key-gen")
+	CT (CKM_SHA3_224, "sha3-224")
+	CT (CKM_SHA3_224_HMAC, "sha3-224-hmac")
+	CT (CKM_SHA3_224_HMAC_GENERAL, "sha3-224-hmac-general")
+	CT (CKM_SHA3_224_KEY_GEN, "sha3-224-key-gen")
+	CT (CKM_SHA3_384, "sha3-384")
+	CT (CKM_SHA3_384_HMAC, "sha3-384-hmac")
+	CT (CKM_SHA3_384_HMAC_GENERAL, "sha3-384-hmac-general")
+	CT (CKM_SHA3_384_KEY_GEN, "sha3-384-key-gen")
+	CT (CKM_SHA3_512, "sha3-512")
+	CT (CKM_SHA3_512_HMAC, "sha3-512-hmac")
+	CT (CKM_SHA3_512_HMAC_GENERAL, "sha3-512-hmac-general")
+	CT (CKM_SHA3_512_KEY_GEN, "sha3-512-key-gen")
 	CT (CKM_CAST_KEY_GEN, "cast-key-gen")
 	CT (CKM_CAST_ECB, "cast-ecb")
 	CT (CKM_CAST_CBC, "cast-cbc")
@@ -543,6 +701,19 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_SHA256_KEY_DERIVATION, "sha256-key-derivation")
 	CT (CKM_SHA384_KEY_DERIVATION, "sha384-key-derivation")
 	CT (CKM_SHA512_KEY_DERIVATION, "sha512-key-derivation")
+	CT (CKM_SHA224_KEY_DERIVATION, "sha224-key-derivation")
+	/* CT (CKM_SHA3_256_KEY_DERIVATION) */
+	CT (CKM_SHA3_256_KEY_DERIVE, "sha3-256-key-derive")
+	/* CT (CKM_SHA3_224_KEY_DERIVATION) */
+	CT (CKM_SHA3_224_KEY_DERIVE, "sha3-224-key-derive")
+	/* CT (CKM_SHA3_384_KEY_DERIVATION) */
+	CT (CKM_SHA3_384_KEY_DERIVE, "sha3-384-key-derive")
+	/* CT (CKM_SHA3_512_KEY_DERIVATION) */
+	CT (CKM_SHA3_512_KEY_DERIVE, "sha3-512-key-derive")
+	/* CT (CKM_SHAKE_128_KEY_DERIVATION) */
+	CT (CKM_SHAKE_128_KEY_DERIVE, "shake-128-key-derive")
+	/* CT (CKM_SHAKE_256_KEY_DERIVATION) */
+	CT (CKM_SHAKE_256_KEY_DERIVE, "shake-256-key-derive")
 	CT (CKM_PBE_MD2_DES_CBC, "pbe-md2-des-cbc")
 	CT (CKM_PBE_MD5_DES_CBC, "pbe-md5-des-cbc")
 	CT (CKM_PBE_MD5_CAST_CBC, "pbe-md5-cast-cbc")
@@ -557,6 +728,9 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_PBE_SHA1_DES2_EDE_CBC, "pbe-sha1-des2-ede-cbc")
 	CT (CKM_PBE_SHA1_RC2_128_CBC, "pbe-sha1-rc2-128-cbc")
 	CT (CKM_PBE_SHA1_RC2_40_CBC, "pbe-sha1-rc2-40-cbc")
+	CT (CKM_SP800_108_COUNTER_KDF, "sp800-108-counter-kdf")
+	CT (CKM_SP800_108_FEEDBACK_KDF, "sp800-108-feedback-kdf")
+	CT (CKM_SP800_108_DOUBLE_PIPELINE_KDF, "sp800-108-double-pipeline-kdf")
 	CT (CKM_PKCS5_PBKD2, "pkcs5-pbkd2")
 	CT (CKM_PBA_SHA1_WITH_SHA1_HMAC, "pba-sha1-with-sha1-hmac")
 	CT (CKM_WTLS_PRE_MASTER_KEY_GEN, "wtls-pre-master-key-gen")
@@ -565,9 +739,47 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_WTLS_PRF, "wtls-prf")
 	CT (CKM_WTLS_SERVER_KEY_AND_MAC_DERIVE, "wtls-server-key-and-mac-derive")
 	CT (CKM_WTLS_CLIENT_KEY_AND_MAC_DERIVE, "wtls-client-key-and-mac-derive")
+	CT (CKM_TLS10_MAC_SERVER, "tls10-mac-server")
+	CT (CKM_TLS10_MAC_CLIENT, "tls10-mac-client")
+	CT (CKM_TLS12_MAC, "tls12-mac")
+	CT (CKM_TLS12_KDF, "tls12-kdf")
+	CT (CKM_TLS12_MASTER_KEY_DERIVE, "tls12-master-key-derive")
+	CT (CKM_TLS12_KEY_AND_MAC_DERIVE, "tls12-key-and-mac-derive")
+	CT (CKM_TLS12_MASTER_KEY_DERIVE_DH, "tls12-master-key-derive-dh")
+	CT (CKM_TLS12_KEY_SAFE_DERIVE, "tls12-key-safe-derive")
+	CT (CKM_TLS_MAC, "tls-mac")
+	CT (CKM_TLS_KDF, "tls-kdf")
 	CT (CKM_KEY_WRAP_LYNKS, "key-wrap-lynks")
 	CT (CKM_KEY_WRAP_SET_OAEP, "key-wrap-set-oaep")
 	CT (CKM_CMS_SIG, "cms-sig")
+	CT (CKM_KIP_DERIVE, "kip-derive")
+	CT (CKM_KIP_WRAP, "kip-wrap")
+	CT (CKM_KIP_MAC, "kip-mac")
+	CT (CKM_CAMELLIA_KEY_GEN, "camellia-key-gen")
+	CT (CKM_CAMELLIA_ECB, "camellia-ecb")
+	CT (CKM_CAMELLIA_CBC, "camellia-cbc")
+	CT (CKM_CAMELLIA_MAC, "camellia-mac")
+	CT (CKM_CAMELLIA_MAC_GENERAL, "camellia-mac-general")
+	CT (CKM_CAMELLIA_CBC_PAD, "camellia-cbc-pad")
+	CT (CKM_CAMELLIA_ECB_ENCRYPT_DATA, "camellia-ecb-encrypt-data")
+	CT (CKM_CAMELLIA_CBC_ENCRYPT_DATA, "camellia-cbc-encrypt-data")
+	CT (CKM_CAMELLIA_CTR, "camellia-ctr")
+	CT (CKM_ARIA_KEY_GEN, "aria-key-gen")
+	CT (CKM_ARIA_ECB, "aria-ecb")
+	CT (CKM_ARIA_CBC, "aria-cbc")
+	CT (CKM_ARIA_MAC, "aria-mac")
+	CT (CKM_ARIA_MAC_GENERAL, "aria-mac-general")
+	CT (CKM_ARIA_CBC_PAD, "aria-cbc-pad")
+	CT (CKM_ARIA_ECB_ENCRYPT_DATA, "aria-ecb-encrypt-data")
+	CT (CKM_ARIA_CBC_ENCRYPT_DATA, "aria-cbc-encrypt-data")
+	CT (CKM_SEED_KEY_GEN, "seed-key-gen")
+	CT (CKM_SEED_ECB, "seed-ecb")
+	CT (CKM_SEED_CBC, "seed-cbc")
+	CT (CKM_SEED_MAC, "seed-mac")
+	CT (CKM_SEED_MAC_GENERAL, "seed-mac-general")
+	CT (CKM_SEED_CBC_PAD, "seed-cbc-pad")
+	CT (CKM_SEED_ECB_ENCRYPT_DATA, "seed-ecb-encrypt-data")
+	CT (CKM_SEED_CBC_ENCRYPT_DATA, "seed-cbc-encrypt-data")
 	CT (CKM_SKIPJACK_KEY_GEN, "skipjack-key-gen")
 	CT (CKM_SKIPJACK_ECB64, "skipjack-ecb64")
 	CT (CKM_SKIPJACK_CBC64, "skipjack-cbc64")
@@ -593,9 +805,22 @@ const p11_constant p11_constant_mechanisms[] = {
 	/* CT (CKM_EC_KEY_PAIR_GEN) */
 	CT (CKM_ECDSA, "ecdsa")
 	CT (CKM_ECDSA_SHA1, "ecdsa-sha1")
+	CT (CKM_ECDSA_SHA224, "ecdsa-sha224")
+	CT (CKM_ECDSA_SHA256, "ecdsa-sha256")
+	CT (CKM_ECDSA_SHA384, "ecdsa-sha384")
+	CT (CKM_ECDSA_SHA512, "ecdsa-sha512")
+	CT (CKM_ECDSA_SHA3_224, "ecdsa-sha3-224")
+	CT (CKM_ECDSA_SHA3_256, "ecdsa-sha3-256")
+	CT (CKM_ECDSA_SHA3_384, "ecdsa-sha3-384")
+	CT (CKM_ECDSA_SHA3_512, "ecdsa-sha3-512")
 	CT (CKM_ECDH1_DERIVE, "ecdh1-derive")
 	CT (CKM_ECDH1_COFACTOR_DERIVE, "ecdh1-cofactor-derive")
 	CT (CKM_ECMQV_DERIVE, "ecmqv-derive")
+	CT (CKM_ECDH_AES_KEY_WRAP, "ecdh-aes-key-wrap")
+	CT (CKM_RSA_AES_KEY_WRAP, "rsa-aes-key-wrap")
+	CT (CKM_EC_EDWARDS_KEY_PAIR_GEN, "ec-edwards-key-pair-gen")
+	CT (CKM_EC_MONTGOMERY_KEY_PAIR_GEN, "ec-montgomery-key-pair-gen")
+	CT (CKM_EDDSA, "eddsa")
 	CT (CKM_JUNIPER_KEY_GEN, "juniper-key-gen")
 	CT (CKM_JUNIPER_ECB128, "juniper-ecb128")
 	CT (CKM_JUNIPER_CBC128, "juniper-cbc128")
@@ -603,25 +828,119 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_JUNIPER_SHUFFLE, "juniper-shuffle")
 	CT (CKM_JUNIPER_WRAP, "juniper-wrap")
 	CT (CKM_FASTHASH, "fasthash")
+	CT (CKM_AES_XTS, "aes-xts")
+	CT (CKM_AES_XTS_KEY_GEN, "aes-xts-key-gen")
 	CT (CKM_AES_KEY_GEN, "aes-key-gen")
 	CT (CKM_AES_ECB, "aes-ecb")
 	CT (CKM_AES_CBC, "aes-cbc")
 	CT (CKM_AES_MAC, "aes-mac")
 	CT (CKM_AES_MAC_GENERAL, "aes-mac-general")
 	CT (CKM_AES_CBC_PAD, "aes-cbc-pad")
+	CT (CKM_AES_CTR, "aes-ctr")
+	CT (CKM_AES_GCM, "aes-gcm")
+	CT (CKM_AES_CCM, "aes-ccm")
+	CT (CKM_AES_CTS, "aes-cts")
+	CT (CKM_AES_CMAC, "aes-cmac")
+	CT (CKM_AES_CMAC_GENERAL, "aes-cmac-general")
+	CT (CKM_AES_XCBC_MAC, "aes-xcbc-mac")
+	CT (CKM_AES_XCBC_MAC_96, "aes-xcbc-mac-96")
+	CT (CKM_AES_GMAC, "aes-gmac")
 	CT (CKM_BLOWFISH_KEY_GEN, "blowfish-key-gen")
 	CT (CKM_BLOWFISH_CBC, "blowfish-cbc")
 	CT (CKM_TWOFISH_KEY_GEN, "twofish-key-gen")
 	CT (CKM_TWOFISH_CBC, "twofish-cbc")
+	CT (CKM_BLOWFISH_CBC_PAD, "blowfish-cbc-pad")
+	CT (CKM_TWOFISH_CBC_PAD, "twofish-cbc-pad")
 	CT (CKM_DES_ECB_ENCRYPT_DATA, "des-ecb-encrypt-data")
 	CT (CKM_DES_CBC_ENCRYPT_DATA, "des-cbc-encrypt-data")
 	CT (CKM_DES3_ECB_ENCRYPT_DATA, "des3-ecb-encrypt-data")
 	CT (CKM_DES3_CBC_ENCRYPT_DATA, "des3-cbc-encrypt-data")
 	CT (CKM_AES_ECB_ENCRYPT_DATA, "aes-ecb-encrypt-data")
 	CT (CKM_AES_CBC_ENCRYPT_DATA, "aes-cbc-encrypt-data")
+	CT (CKM_GOSTR3410_KEY_PAIR_GEN, "gostr3410-key-pair-gen")
+	CT (CKM_GOSTR3410, "gostr3410")
+	CT (CKM_GOSTR3410_WITH_GOSTR3411, "gostr3410-with-gostr3411")
+	CT (CKM_GOSTR3410_KEY_WRAP, "gostr3410-key-wrap")
+	CT (CKM_GOSTR3410_DERIVE, "gostr3410-derive")
+	CT (CKM_GOSTR3411, "gostr3411")
+	CT (CKM_GOSTR3411_HMAC, "gostr3411-hmac")
+	CT (CKM_GOST28147_KEY_GEN, "gost28147-key-gen")
+	CT (CKM_GOST28147_ECB, "gost28147-ecb")
+	CT (CKM_GOST28147, "gost28147")
+	CT (CKM_GOST28147_MAC, "gost28147-mac")
+	CT (CKM_GOST28147_KEY_WRAP, "gost28147-key-wrap")
+	CT (CKM_CHACHA20_KEY_GEN, "chacha20-key-gen")
+	CT (CKM_CHACHA20, "chacha20")
+	CT (CKM_POLY1305_KEY_GEN, "poly1305-key-gen")
+	CT (CKM_POLY1305, "poly1305")
+	CT (CKM_EC_KEY_PAIR_GEN_W_EXTRA_BITS, "ec-key-pair-gen-w-extra-bits")
 	CT (CKM_DSA_PARAMETER_GEN, "dsa-parameter-gen")
 	CT (CKM_DH_PKCS_PARAMETER_GEN, "dh-pkcs-parameter-gen")
 	CT (CKM_X9_42_DH_PARAMETER_GEN, "x9-42-dh-parameter-gen")
+	/* CT (CKM_DSA_PROBABLISTIC_PARAMETER_GEN) */
+	CT (CKM_DSA_PROBABILISTIC_PARAMETER_GEN, "dsa-probabilistic-parameter-gen")
+	CT (CKM_DSA_SHAWE_TAYLOR_PARAMETER_GEN, "dsa-shawe-taylor-parameter-gen")
+	CT (CKM_DSA_FIPS_G_GEN, "dsa-fips-g-gen")
+	CT (CKM_AES_OFB, "aes-ofb")
+	CT (CKM_AES_CFB64, "aes-cfb64")
+	CT (CKM_AES_CFB8, "aes-cfb8")
+	CT (CKM_AES_CFB128, "aes-cfb128")
+	CT (CKM_AES_CFB1, "aes-cfb1")
+	CT (CKM_AES_KEY_WRAP, "aes-key-wrap")
+	CT (CKM_AES_KEY_WRAP_PAD, "aes-key-wrap-pad")
+	CT (CKM_AES_KEY_WRAP_KWP, "aes-key-wrap-kwp")
+	CT (CKM_AES_KEY_WRAP_PKCS7, "aes-key-wrap-pkcs7")
+	CT (CKM_RSA_PKCS_TPM_1_1, "rsa-pkcs-tpm-1-1")
+	CT (CKM_RSA_PKCS_OAEP_TPM_1_1, "rsa-pkcs-oaep-tpm-1-1")
+	CT (CKM_SHA_1_KEY_GEN, "sha-1-key-gen")
+	CT (CKM_SHA224_KEY_GEN, "sha224-key-gen")
+	CT (CKM_SHA256_KEY_GEN, "sha256-key-gen")
+	CT (CKM_SHA384_KEY_GEN, "sha384-key-gen")
+	CT (CKM_SHA512_KEY_GEN, "sha512-key-gen")
+	CT (CKM_SHA512_224_KEY_GEN, "sha512-224-key-gen")
+	CT (CKM_SHA512_256_KEY_GEN, "sha512-256-key-gen")
+	CT (CKM_SHA512_T_KEY_GEN, "sha512-t-key-gen")
+	CT (CKM_NULL, "null")
+	CT (CKM_BLAKE2B_160, "blake2b-160")
+	CT (CKM_BLAKE2B_160_HMAC, "blake2b-160-hmac")
+	CT (CKM_BLAKE2B_160_HMAC_GENERAL, "blake2b-160-hmac-general")
+	CT (CKM_BLAKE2B_160_KEY_DERIVE, "blake2b-160-key-derive")
+	CT (CKM_BLAKE2B_160_KEY_GEN, "blake2b-160-key-gen")
+	CT (CKM_BLAKE2B_256, "blake2b-256")
+	CT (CKM_BLAKE2B_256_HMAC, "blake2b-256-hmac")
+	CT (CKM_BLAKE2B_256_HMAC_GENERAL, "blake2b-256-hmac-general")
+	CT (CKM_BLAKE2B_256_KEY_DERIVE, "blake2b-256-key-derive")
+	CT (CKM_BLAKE2B_256_KEY_GEN, "blake2b-256-key-gen")
+	CT (CKM_BLAKE2B_384, "blake2b-384")
+	CT (CKM_BLAKE2B_384_HMAC, "blake2b-384-hmac")
+	CT (CKM_BLAKE2B_384_HMAC_GENERAL, "blake2b-384-hmac-general")
+	CT (CKM_BLAKE2B_384_KEY_DERIVE, "blake2b-384-key-derive")
+	CT (CKM_BLAKE2B_384_KEY_GEN, "blake2b-384-key-gen")
+	CT (CKM_BLAKE2B_512, "blake2b-512")
+	CT (CKM_BLAKE2B_512_HMAC, "blake2b-512-hmac")
+	CT (CKM_BLAKE2B_512_HMAC_GENERAL, "blake2b-512-hmac-general")
+	CT (CKM_BLAKE2B_512_KEY_DERIVE, "blake2b-512-key-derive")
+	CT (CKM_BLAKE2B_512_KEY_GEN, "blake2b-512-key-gen")
+	CT (CKM_SALSA20, "salsa20")
+	CT (CKM_CHACHA20_POLY1305, "chacha20-poly1305")
+	CT (CKM_SALSA20_POLY1305, "salsa20-poly1305")
+	CT (CKM_X3DH_INITIALIZE, "x3dh-initialize")
+	CT (CKM_X3DH_RESPOND, "x3dh-respond")
+	CT (CKM_X2RATCHET_INITIALIZE, "x2ratchet-initialize")
+	CT (CKM_X2RATCHET_RESPOND, "x2ratchet-respond")
+	CT (CKM_X2RATCHET_ENCRYPT, "x2ratchet-encrypt")
+	CT (CKM_X2RATCHET_DECRYPT, "x2ratchet-decrypt")
+	CT (CKM_XEDDSA, "xeddsa")
+	CT (CKM_HKDF_DERIVE, "hkdf-derive")
+	CT (CKM_HKDF_DATA, "hkdf-data")
+	CT (CKM_HKDF_KEY_GEN, "hkdf-key-gen")
+	CT (CKM_SALSA20_KEY_GEN, "salsa20-key-gen")
+	CT (CKM_IKE2_PRF_PLUS_DERIVE, "ike2-prf-plus-derive")
+	CT (CKM_IKE_PRF_DERIVE, "ike-prf-derive")
+	CT (CKM_IKE1_PRF_DERIVE, "ike1-prf-derive")
+	CT (CKM_IKE1_EXTENDED_DERIVE, "ike1-extended-derive")
+	CT (CKM_HSS_KEY_PAIR_GEN, "hss-key-pair-gen")
+	CT (CKM_HSS, "hss")
 	CT (CKM_IBM_SHA3_224, "ibm-sha3-224")
 	CT (CKM_IBM_SHA3_256, "ibm-sha3-256")
 	CT (CKM_IBM_SHA3_384, "ibm-sha3-384")
@@ -632,11 +951,14 @@ const p11_constant p11_constant_mechanisms[] = {
 	CT (CKM_IBM_EC_X448, "ibm-ec-x448")
 	CT (CKM_IBM_ED448_SHA3, "ibm-ed448-sha3")
 	CT (CKM_IBM_DILITHIUM, "ibm-dilithium")
+	CT (CKM_IBM_KYBER, "ibm-kyber")
 	CT (CKM_IBM_SHA3_224_HMAC, "ibm-sha3-224-hmac")
 	CT (CKM_IBM_SHA3_256_HMAC, "ibm-sha3-256-hmac")
 	CT (CKM_IBM_SHA3_384_HMAC, "ibm-sha3-384-hmac")
 	CT (CKM_IBM_SHA3_512_HMAC, "ibm-sha3-512-hmac")
+	CT (CKM_IBM_ECDSA_OTHER, "ibm-ecdsa-other")
 	CT (CKM_IBM_ATTRIBUTEBOUND_WRAP, "ibm-attributebound-wrap")
+	CT (CKM_IBM_BTC_DERIVE, "ibm-btc-derive")
 	{ CKA_INVALID },
 };
 
@@ -649,10 +971,13 @@ const p11_constant p11_constant_hw_features[] = {
 };
 
 const p11_constant p11_constant_profiles[] = {
+	CT (CKP_INVALID_ID, "invalid-id")
 	CT (CKP_BASELINE_PROVIDER, "baseline-provider")
 	CT (CKP_EXTENDED_PROVIDER, "extended-provider")
 	CT (CKP_AUTHENTICATION_TOKEN, "authentication-token")
 	CT (CKP_PUBLIC_CERTIFICATES_TOKEN, "public-certificates-token")
+	CT (CKP_COMPLETE_PROVIDER, "complete-provider")
+	CT (CKP_HKDF_TLS_TOKEN, "hkdf-tls-token")
 	CT (CKP_VENDOR_DEFINED, "vendor-defined")
 	{ CKA_INVALID },
 };
