@@ -980,7 +980,7 @@ p11_kit_iter_destroy_object (P11KitIter *iter)
 /**
  * p11_kit_iter_get_attributes:
  * @iter: the iterator
- * @template: (array length=count) (inout): the attributes to get
+ * @templ: (array length=count) (inout): the attributes to get
  * @count: the number of attributes
  *
  * Get attributes for the current matching object.
@@ -999,7 +999,7 @@ p11_kit_iter_destroy_object (P11KitIter *iter)
  */
 CK_RV
 p11_kit_iter_get_attributes (P11KitIter *iter,
-                             CK_ATTRIBUTE *template,
+                             CK_ATTRIBUTE *templ,
                              CK_ULONG count)
 {
 	return_val_if_fail (iter != NULL, CKR_GENERAL_ERROR);
@@ -1009,13 +1009,59 @@ p11_kit_iter_get_attributes (P11KitIter *iter,
 	return_val_if_fail (iter->object != 0, CKR_GENERAL_ERROR);
 
 	return (iter->module->C_GetAttributeValue) (iter->session, iter->object,
-	                                            template, count);
+	                                            templ, count);
+}
+
+static CK_RV
+prepare_recursive_attribute (P11KitIter *iter,
+			     CK_ATTRIBUTE *attr,
+			     CK_ATTRIBUTE *templ,
+			     CK_ULONG templ_len)
+{
+	CK_RV rv;
+	CK_ULONG i;
+
+	return_val_if_fail (iter != NULL, CKR_GENERAL_ERROR);
+	return_val_if_fail (attr != NULL, CKR_GENERAL_ERROR);
+	return_val_if_fail (templ != NULL, CKR_GENERAL_ERROR);
+	return_val_if_fail (templ_len != 0, CKR_GENERAL_ERROR);
+	return_val_if_fail (IS_ATTRIBUTE_ARRAY (attr), CKR_GENERAL_ERROR);
+
+	memset (templ, 0, templ_len);
+	rv = (iter->module->C_GetAttributeValue) (iter->session, iter->object, attr, 1);
+
+	switch (rv) {
+	case CKR_OK:
+	case CKR_ATTRIBUTE_TYPE_INVALID:
+	case CKR_ATTRIBUTE_SENSITIVE:
+		break;
+	default:
+		return_val_if_fail (rv != CKR_BUFFER_TOO_SMALL, rv);
+		return rv;
+	}
+
+	for (i = 0; i < templ_len / sizeof (CK_ATTRIBUTE); ++i) {
+		return_val_if_fail (templ[i].type != CKA_INVALID, CKR_GENERAL_ERROR);
+		return_val_if_fail (templ[i].ulValueLen != 0, CKR_GENERAL_ERROR);
+		return_val_if_fail (templ[i].ulValueLen != (CK_ULONG)-1, CKR_GENERAL_ERROR);
+
+		templ[i].pValue = malloc (templ[i].ulValueLen);
+		return_val_if_fail (templ[i].pValue != NULL, CKR_HOST_MEMORY);
+
+		if (IS_ATTRIBUTE_ARRAY (templ + i)) {
+                        rv = prepare_recursive_attribute (iter, attr, templ[i].pValue,
+							  templ[i].ulValueLen);
+			return_val_if_fail (rv == CKR_OK, rv);
+                }
+	}
+
+	return CKR_OK;
 }
 
 /**
  * p11_kit_iter_load_attributes:
  * @iter: the iterator
- * @template: (array length=count) (inout): the attributes to load
+ * @templ: (array length=count) (inout): the attributes to load
  * @count: the number of attributes
  *
  * Retrieve attributes for the current matching object.
@@ -1043,7 +1089,7 @@ p11_kit_iter_get_attributes (P11KitIter *iter,
  */
 CK_RV
 p11_kit_iter_load_attributes (P11KitIter *iter,
-                              CK_ATTRIBUTE *template,
+                              CK_ATTRIBUTE *templ,
                               CK_ULONG count)
 {
 	CK_ATTRIBUTE *original = NULL;
@@ -1059,13 +1105,13 @@ p11_kit_iter_load_attributes (P11KitIter *iter,
 	if (count == 0)
 		return CKR_OK;
 
-	original = memdup (template, count * sizeof (CK_ATTRIBUTE));
+	original = memdup (templ, count * sizeof (CK_ATTRIBUTE));
 	return_val_if_fail (original != NULL, CKR_HOST_MEMORY);
 
 	for (i = 0; i < count; i++)
-		template[i].pValue = NULL;
+		templ[i].pValue = NULL;
 
-	rv = (iter->module->C_GetAttributeValue) (iter->session, iter->object, template, count);
+	rv = (iter->module->C_GetAttributeValue) (iter->session, iter->object, templ, count);
 
 	switch (rv) {
 	case CKR_OK:
@@ -1079,23 +1125,33 @@ p11_kit_iter_load_attributes (P11KitIter *iter,
 	}
 
 	for (i = 0; i < count; i++) {
-		if (template[i].ulValueLen == (CK_ULONG)-1 ||
-		    template[i].ulValueLen == 0) {
-			free (original[i].pValue);
+		if (templ[i].ulValueLen == (CK_ULONG)-1 ||
+		    templ[i].ulValueLen == 0) {
+			p11_attr_clear (original + i);
 
 		} else if (original[i].pValue != NULL &&
-		           template[i].ulValueLen == original[i].ulValueLen) {
-			template[i].pValue = original[i].pValue;
+		           templ[i].ulValueLen == original[i].ulValueLen) {
+			templ[i].pValue = original[i].pValue;
 
 		} else {
-			template[i].pValue = realloc (original[i].pValue, template[i].ulValueLen);
-			return_val_if_fail (template[i].pValue != NULL, CKR_HOST_MEMORY);
+			templ[i].pValue = realloc (original[i].pValue, templ[i].ulValueLen);
+			return_val_if_fail (templ[i].pValue != NULL, CKR_HOST_MEMORY);
+
+			if (IS_ATTRIBUTE_ARRAY (templ + i)) {
+				rv = prepare_recursive_attribute (iter, templ + i,
+								  templ[i].pValue,
+								  templ[i].ulValueLen);
+				if (rv != CKR_OK) {
+					free (original);
+					return rv;
+				}
+			}
 		}
 	}
 
 	free (original);
 
-	rv = (iter->module->C_GetAttributeValue) (iter->session, iter->object, template, count);
+	rv = (iter->module->C_GetAttributeValue) (iter->session, iter->object, templ, count);
 
 	switch (rv) {
 	case CKR_OK:
@@ -1109,10 +1165,10 @@ p11_kit_iter_load_attributes (P11KitIter *iter,
 	}
 
 	for (i = 0; i < count; i++) {
-		if (template[i].ulValueLen == (CK_ULONG)-1 ||
-		    template[i].ulValueLen == 0) {
-			free (template[i].pValue);
-			template[i].pValue = NULL;
+		if (templ[i].ulValueLen == (CK_ULONG)-1 ||
+		    templ[i].ulValueLen == 0) {
+			free (templ[i].pValue);
+			templ[i].pValue = NULL;
 		}
 	}
 
