@@ -731,7 +731,11 @@ log_token_info (p11_buffer *buf,
 		          (unsigned int)info->firmwareVersion.minor);
 		p11_buffer_add (buf, temp, -1);
 		p11_buffer_add (buf, "\n\tutcTime: ", -1);
-		p11_buffer_add (buf, (info->flags & CKF_CLOCK_ON_TOKEN) ? (const char*)info->utcTime : "", sizeof (info->utcTime));
+		if (info->flags & CKF_CLOCK_ON_TOKEN) {
+			p11_buffer_add (buf, (const char*)info->utcTime, sizeof (info->utcTime));
+		} else {
+			p11_buffer_add (buf, "(not supported)", -1);
+		}
 		p11_buffer_add (buf, "\n      }\n", -1);
 	}
 }
