@@ -42,8 +42,8 @@ test_generate_keypair_ecdsa() {
 		if "$abs_top_builddir"/p11-kit/p11-kit-testable generate-keypair --login --label="ecdsa-$curve" --type=ecdsa --curve="$curve" "pkcs11:token=PUBKEY%20LABEL?pin-value=booo" 2> err.out; then
 			assert_fail "expected to fail: p11-kit generate-keypair"
 		fi
+		assert_contains err.out "key-pair generation failed: The crypto mechanism is invalid or unrecognized"
 	done
-	assert_contains err.out "key-pair generation failed: The crypto mechanism is invalid or unrecognized"
 
 	if "$abs_top_builddir"/p11-kit/p11-kit-testable generate-keypair --login --label="ecdsa-unknown" --type=ecdsa --curve=unknown "pkcs11:token=PUBKEY%20LABEL?pin-value=booo" 2> err.out; then
 		assert_fail "p11-kit generate-keypair succeeded for unknown ecdsa curve"
@@ -54,16 +54,16 @@ test_generate_keypair_ecdsa() {
 test_generate_keypair_eddsa() {
 	for curve in ed25519 ed448; do
 		if "$abs_top_builddir"/p11-kit/p11-kit-testable generate-keypair --login --label="eddsa-$curve" --type=eddsa --curve="$curve" "pkcs11:token=PUBKEY%20LABEL?pin-value=booo" 2> err.out; then
-			assert_fail "unable to run: p11-kit generate-keypair"
+			assert_fail "expected to fail: p11-kit generate-keypair"
 		fi
+		assert_contains err.out "key-pair generation failed: The crypto mechanism is invalid or unrecognized"
 	done
-	assert_contains err.out "key-pair generation failed: The crypto mechanism is invalid or unrecognized"
 
-	if "$abs_top_builddir"/p11-kit/p11-kit-testable generate-keypair --login --label="eddsa-unknown" --type=eddsa --curve=unknown "pkcs11:token=PUBKEY%20LABEL?pin-value=booo"; then
+	if "$abs_top_builddir"/p11-kit/p11-kit-testable generate-keypair --login --label="eddsa-unknown" --type=eddsa --curve=unknown "pkcs11:token=PUBKEY%20LABEL?pin-value=booo" 2> err.out; then
 		assert_fail "p11-kit generate-keypair succeeded for unknown eddsa curve"
 	fi
 	assert_contains err.out "unknown curve name: unknown"
 }
 
 run test_generate_keypair_mock test_generate_keypair_rsa \
-    test_generate_keypair_ecdsa test_generate_keypair_ecdsa
+    test_generate_keypair_ecdsa test_generate_keypair_eddsa
