@@ -45,7 +45,7 @@
 #include "debug.h"
 #include "message.h"
 #include "path.h"
-#include "tool.h"
+#include "options.h"
 
 #include <assert.h>
 #include <ctype.h>
